package com.ngrob.android.bluemoon.core.datastore

import android.util.Log
import androidx.datastore.core.DataStore
import com.ngrob.android.bluemoon.core.model.PillRecipe
import com.ngrob.android.bluemoon.core.model.UserData
import kotlinx.coroutines.flow.map
import java.io.IOException
import java.time.LocalDate
import java.time.format.DateTimeFormatter
import javax.inject.Inject

class BluemoonPreferencesDataSource @Inject constructor(
    private val userPreferences: DataStore<UserPreferences>,
) {

    val userData = userPreferences.data.map {
        UserData(
            trackBirthControl = it.trackBirthControl,
            pillRecipe =
            if (it.hasPillRecipe()) PillRecipe(
                numberOfPills = it.pillRecipe.numberOfPills,
                firstIntake = LocalDate.parse(
                    it.pillRecipe.pillFirstIntake,
                    DateTimeFormatter.ISO_LOCAL_DATE
                )
            ) else
                PillRecipe(
                    numberOfPills = 24,
                    firstIntake = LocalDate.now()
                )
        )

    }

    suspend fun setPillRecipe(pillRecipe: PillRecipe) {
        try {
            userPreferences.updateData {
                it.copy {

                    this.pillRecipe = PillRecipeProto.getDefaultInstance().copy {
                        this.pillFirstIntake =
                            pillRecipe.firstIntake.format(DateTimeFormatter.ISO_LOCAL_DATE)
                        this.numberOfPills = pillRecipe.numberOfPills
                    }

                }
            }
        } catch (
            ioException: IOException
        ) {
            Log.e("BluemoonPreferences", "Failed to update user preferences", ioException)
        }
    }

    suspend fun setTrackBirthControl(trackBirthControl: Boolean) {
        try {
            userPreferences.updateData {
                it.copy {
                    this.trackBirthControl = trackBirthControl
                }
            }
        } catch (
            ioException: IOException
        ) {
            Log.e("BluemoonPreferences", "Failed to update user preferences", ioException)
        }
    }
}

